/**
 * OW2 Util
 * Copyright (C) 2008-2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: IEjbJarMethodMetadata.java 6271 2012-07-10 11:10:15Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.api;

import java.util.List;
import java.util.Map;

import javax.ejb.Remove;
import javax.ejb.TransactionAttributeType;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.ICommonMethodMetadata;
import org.ow2.util.ee.metadata.common.api.interfaces.IAnnotationSecurityPermitAll;
import org.ow2.util.ee.metadata.common.api.interfaces.IAnnotationSecurityRolesAllowed;
import org.ow2.util.ee.metadata.common.api.interfaces.IEJBInterceptors;
import org.ow2.util.ee.metadata.common.api.interfaces.IInterceptorExcludeDefaultInterceptors;
import org.ow2.util.ee.metadata.common.api.interfaces.ITransactionAttribute;
import org.ow2.util.ee.metadata.common.api.interfaces.IWebMethod;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbSchedule;

/**
 * Interface of the Method Metadata for EJB.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Gael Lalire
 */
public interface IEjbJarMethodMetadata<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        extends ICommonMethodMetadata<C, M, F>,
        ITransactionAttribute,
        IEJBInterceptors,
        ILock,
        IEjbAccessTimeout,
        IEjbSchedule,
        IWebMethod,
        IAnnotationSecurityRolesAllowed,
        IAnnotationSecurityPermitAll,
        IInterceptorExcludeDefaultInterceptors {

    /**
     * @return name of the method
     */
    String getMethodName();

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    TransactionAttributeType getTransactionAttributeType();

    /**
     * Sets Transaction Attribute Level.
     * @param transactionAttributeLevel the level of transaction.
     */
    void setTransactionAttributeLevel(final TransactionAttributeLevel transactionAttributeLevel);

    /**
     * @return transaction Attribute Level.
     */
    TransactionAttributeLevel getTransactionAttributeLevel();

    /**
     * Sets Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType);

    /**
     * @return true if the method is a business method.
     */
    boolean isBusinessMethod();

    /**
     * This method is a business method.
     * @param flag true/false if method is a business method.
     */
    void setBusinessMethod(final boolean flag);

    /**
     * @return true if the method is a lifecycle method.
     */
    boolean isLifeCycleMethod();

    /**
     * @return remove attributes  &#64;{@link javax.ejb.Remove} attributes
     */
    Remove getJRemove();

    /**
     * Sets &#64;{@link javax.ejb.Remove} attribute.
     * @param remove contains the attribute with retainIfException.
     */
    void setRemove(final Remove remove);

    /**
     * @return true if method has  &#64;{@link javax.ejb.PostActivate}
     */
    boolean isPostActivate();

    /**
     * Sets true if method has &#64;{@link javax.ejb.PostActivate}.
     * @param postActivate true/false.
     */
    void setPostActivate(final boolean postActivate);

    /**
     * @return true if method has &#64;{@link javax.ejb.PrePassivate}.
     */

    boolean isPrePassivate();

    /**
     * Sets true if method has &#64;{@link javax.ejb.PrePassivate}.
     * @param prePassivate true/false.
     */
    void setPrePassivate(final boolean prePassivate);

    /**
     * @return true if method has &#64;{@link javax.ejb.Timeout}.
     */
    boolean isTimeout();

    /**
     * Sets true if method has &#64;{@link javax.ejb.Timeout}.
     * @param timeout true/false.
     */
    void setTimeout(final boolean timeout);

    /**
     * @return true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     */
    boolean isAroundInvoke();

    /**
     * Sets true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     * @param aroundInvoke true/false
     */
    void setAroundInvoke(final boolean aroundInvoke);

    /**
     * @return true if this method won't use user interceptors.
     */
    boolean isExcludedClassInterceptors();

    /**
     * Flag this method as a method which exclude user interceptors.
     * @param excludeClassInterceptors true if this method is a method which exclude user interceptors.
     */
    void setExcludeClassInterceptors(final boolean excludeClassInterceptors);

    /**
     * @return object representing list of &#64;{@link javax.interceptor.Interceptors}.
     */
    IJInterceptors getAnnotationInterceptors();

    /**
     * Sets the object representing the &#64;{@link javax.interceptor.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    void setAnnotationsInterceptors(final IJInterceptors annotationInterceptors);

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     * of user interceptors that enhancer will use.
     */
    Map<InterceptorType, List<? extends IJClassInterceptor>> getUserEasyBeansInterceptors();

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param userInterceptors list of interceptors that enhancer will use.
     */
    void setUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> userInterceptors);

    /**
     * @return list of global interceptors that enhancer will use. (ie : Remove interceptor)
     */
    List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors();

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors);

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    void setRolesAllowed(final List<String> rolesAllowed);

    /**
     * @return the list of roles allowed on this class/method.
     */
    List<String> getRolesAllowed();

    /**
     * This method has PermitAll annotation.
     * @param permitAll the boolean value.
     */
    void setPermitAll(final boolean permitAll);

    /**
     * @return true if PermitAll annotation.
     */
    boolean hasPermitAll();

    /**
     * This method has DenyAll annotation.
     * @param denyAll the boolean value.
     */
    void setDenyAll(final boolean denyAll);

    /**
     * @return true if DenyAll annotation.
     */
    boolean hasDenyAll();

    /**
     * @return true if this class won't use default interceptors.
     */
    boolean isExcludedDefaultInterceptors();

    /**
     * Flag this class as a class which exclude default interceptors.
     * @param excludeDefaultInterceptors true if this class is a class which exclude default interceptors.
     */
    void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors);
    
    /**
     * @return true if method is an afterBegin method.
     */
    boolean isAfterBegin();

    /**
     * Sets true if method is an afterBegin method.
     */
    void setAfterBegin();

    /**
     * @return true if method is an AfterCompletion method.
     */
    boolean isAfterCompletion();

    /**
     * Sets true if method is an AfterCompletion method.
     */
    void setAfterCompletion();

    /**
     * @return true if method is an beforeCompletion method.
     */
    boolean isBeforeCompletion();

    /**
     * Sets true if method is an beforeCompletion method.
     */
    void setBeforeCompletion();

}
